'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _index = require('../index');

var _index2 = _interopRequireDefault(_index);

var _options = require('../options');

var _options2 = _interopRequireDefault(_options);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var USAGE = 'Usage: chunkify.interval(Function fn, Number final, [Object options])';

var ok_usage = function ok_usage(fn, final, options) {
  if (!_underscore2['default'].isFunction(fn)) {
    throw new Error(USAGE + ' - bad fn; not a function');
  } else if (!_underscore2['default'].isNumber(final)) {
    throw new Error(USAGE + ' - bad final; not a number');
  }
  var start = options.start;

  if (start != null) {
    if (!_underscore2['default'].isNumber(start)) {
      throw new Error(USAGE + ' - bad start; not a number');
    } else if (start > final) {
      throw new Error(USAGE + ' - bad start; it\'s greater than final');
    }
  }
};

var interval = function interval(fn, final) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  ok_usage(fn, final, options);

  var _$defaults = _underscore2['default'].defaults(options, { start: 0 });

  var start = _$defaults.start;

  var okoptions = _options2['default'].of(options);
  var generator = _index2['default'].generator(start, final, okoptions);
  var process_chunk_sync = function process_chunk_sync(resolve, reject) {
    var next = generator.next();
    while (!(next.value instanceof Promise) && !next.done) {
      try {
        fn.call(okoptions.scope, next.value);
      } catch (error) {
        return reject({ error: error, index: next.value });
      }
      next = generator.next();
    }
    if (next.done) {
      return resolve();
    }
    return next.value.then(function () {
      return process_chunk_sync(resolve, reject);
    });
  };
  return new Promise(process_chunk_sync);
};

exports['default'] = interval;
module.exports = exports['default'];